@@ -2,12 +2,12 @@
 
 from .base import QueryableProperty, queryable_property
 from .cache_behavior import CACHE_RETURN_VALUE, CACHE_VALUE, CLEAR_CACHE, DO_NOTHING
-from .common import RangeCheckProperty, ValueCheckProperty
+from .common import AggregateProperty, RangeCheckProperty, ValueCheckProperty
 from .mixins import AnnotationMixin, boolean_filter, LookupFilterMixin, lookup_filter, SetterMixin, UpdateMixin
 
 __all__ = (
     'QueryableProperty', 'queryable_property',
     'CACHE_RETURN_VALUE', 'CACHE_VALUE', 'CLEAR_CACHE', 'DO_NOTHING',
-    'RangeCheckProperty', 'ValueCheckProperty',
+    'AggregateProperty', 'RangeCheckProperty', 'ValueCheckProperty',
     'AnnotationMixin', 'boolean_filter', 'LookupFilterMixin', 'lookup_filter', 'SetterMixin', 'UpdateMixin',
 )
