@@ -4,11 +4,12 @@ from collections import namedtuple
 from contextlib import contextmanager
 from functools import partial
 
+import six
 from django.utils.tree import Node
 
 from .compat import (
     ADD_Q_METHOD_NAME, ANNOTATION_TO_AGGREGATE_ATTRIBUTES_MAP, BUILD_FILTER_METHOD_NAME, contains_aggregate,
-    convert_build_filter_to_add_q_kwargs, dummy_context, get_related_model, LOOKUP_SEP, NEED_HAVING_METHOD_NAME,
+    convert_build_filter_to_add_q_kwargs, get_related_model, LOOKUP_SEP, NEED_HAVING_METHOD_NAME, nullcontext,
     QUERY_CHAIN_METHOD_NAME
 )
 from .exceptions import FieldDoesNotExist, QueryablePropertyDoesNotExist, QueryablePropertyError
@@ -273,6 +274,18 @@ class QueryablePropertiesQueryMixin(InjectableMixin):
             return self.build_filter(*args, **kwargs)
         return super(QueryablePropertiesQueryMixin, self).add_filter(*args, **kwargs)
 
+    def add_ordering(self, *ordering, **kwargs):
+        for field_name in ordering:
+            # Ordering by a queryable property via simple string values
+            # requires auto-annotating here, while a queryable property used
+            # in a complex ordering expression is resolved through overridden
+            # query methods.
+            if isinstance(field_name, six.string_types) and field_name != '?':
+                if field_name.startswith('-') or field_name.startswith('+'):
+                    field_name = field_name[1:]
+                self._auto_annotate(field_name.split(LOOKUP_SEP))
+        return super(QueryablePropertiesQueryMixin, self).add_ordering(*ordering, **kwargs)
+
     def build_filter(self, filter_expr, *args, **kwargs):
         # Check if the given filter expression is meant to use a queryable
         # property. Therefore, the possibility of filter_expr not being of the
@@ -304,7 +317,7 @@ class QueryablePropertiesQueryMixin(InjectableMixin):
         # the property signals the need of its own annotation to function.
         # If so, add the annotation first to avoid endless recursion, since
         # resolved filter will likely contain the same property name again.
-        context = dummy_context()
+        context = nullcontext()
         if property_ref.property.filter_requires_annotation:
             full_group_by = bool(ANNOTATION_TO_AGGREGATE_ATTRIBUTES_MAP) and not self.select
             context = self._add_queryable_property_annotation(property_ref, full_group_by)
